const express = require("express")
const mongoose = require("mongoose")
const bodyParser = require("body-parser")

const app = express()
app.set("view engine", "ejs")
app.use(express.static(__dirname + "/public"))
app.use(bodyParser.urlencoded({extended: true}))

const port = parseInt(process.env.PORT)

mongoose.connect(`mongodb://${process.env.MONGO_USER}:${process.env.MONGO_PASSWORD}@${process.env.MONGO_URL}/fruits`)

const fruitSchema = new mongoose.Schema({
    fruit: String,
    photo: String,
    count: Number
}, {strict: false} )

const Fruits =mongoose.model("fruits", fruitSchema)

app.get("/", async (req, res) => {
    const fruits = await Fruits.find({})
    res.render("index", {fruits: fruits})
})

app.post("/order", async (req, res) => {
    const amount_raw = req.body.order_amount
    const fruit = req.body.fruit
    try {
        const amount = parseInt(amount_raw)
        const avaliable = await Fruits.findOne({fruit: fruit})
        if (amount > avaliable.count) {
            console.log(`${req.socket.remoteAddress} FAILURE Tried to order ${amount} of ${fruit} when ${avaliable.count} are avaliable`)
            throw new Error()
        }
        await Fruits.updateOne({fruit: fruit},
             {"$inc": {"count": -amount}})
        console.log(`${req.socket.remoteAddress} SUCCESS Ordered ${amount} of ${fruit}`)
    }
    catch (err) {
        console.log(err)
    }
    finally {
        res.redirect("/")
    }
})

app.post("/restock", async (req, res) => {
    const fruit = req.body.fruit
    try {
        await Fruits.updateOne({fruit: fruit},
             {"$inc": {"count": 10}})
        console.log(`${req.socket.remoteAddress} SUCCESS Restocked ${fruit}`)
    }
    catch (err){
        console.log(err)
    }
    finally {
        res.redirect("/")
    }
})


app.listen(port, () => {
    console.log(`App listening on port ${port}`)
})